<?php
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2009 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_1/includes/database.php $
 * $Id: database.php 1453 2009-02-21 21:47:23Z Quietust $
 */

if (!defined('IN_GAME') && !defined('IN_TURNS'))
	die("Access denied");

// Supported database types:
// 'mysql' - MySQL 4.1 or greater
// 'pgsql' - PostgreSQL 8.1 or greater

function db_open ($type, $sock, $host, $port, $user, $pass, $name)
{
	if (!strlen($sock) && !strlen($host))
		die('Invalid database configuration - must specify either hostname or UNIX socket');
	if (strlen($sock) && strlen($host))
		die('Invalid database configuration - cannot specify both hostname and UNIX socket');

	// Select database driver and construct DSN for connection
	switch ($type)
	{
	case 'mysql':
		$dsn = 'mysql:';
		if ($sock)
			$dsn .= 'unix_socket='. $sock .';';
		elseif ($host)
		{
			$dsn .= 'host='. $host .';';
			if ($port)
				$dsn .= 'port='. $port .';';
		}
		$dsn .= 'dbname='. $name;
		break;
	case 'pgsql':
		$dsn = 'pgsql:';
		if ($sock)
			$dsn .= 'host='. $sock .' ';
		elseif ($host)
		{
			$dsn .= 'host='. $host .' ';
			if ($port)
				$dsn .= 'port='. $port .' ';
		}
		$dsn .= 'dbname='. $name;
		break;
	default:
		die('An unsupported database driver has been specified!');
	}

	// Attempt to open database connection
	try
	{
		$db = new PDO($dsn, $user, $pass);
	}
	catch (PDOException $e)
	{
		return NULL;
	}

	$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);

	// Database-specific post-connect actions
	switch ($type)
	{
	case 'mysql':
		$db->query("SET NAMES 'utf8'");
		$db->query("SET SESSION sql_mode=''");
		break;
	}

	return $db;
}

// Shortcut functions for binding different types
// Integer
function db_bindIntValue ($query, $parm, $value)
{
	$query->bindValue($parm, $value, PDO::PARAM_INT);
}
// String
function db_bindStrValue ($query, $parm, $value)
{
	$query->bindValue($parm, $value, PDO::PARAM_STR);
}
// Float
function db_bindFltValue ($query, $parm, $value)
{
	$query->bindValue($parm, $value, PDO::PARAM_STR);
}
// Automatically bind all parameters based on type (for cases where the parameter count varies)
function db_bindAllParms ($query, $parms)
{
	foreach ($parms as $i => $val)
	{
		if (is_numeric($val))
		{
			if ($val - floor($val) > 0.00001)
				db_bindFltValue($query, $i + 1, $val);
			else	db_bindIntValue($query, $i + 1, $val);
		}
		else	db_bindStrValue($query, $i + 1, $val);
	}
}
// Clear the contents of a table and reset its auto-incrementing ID
function db_clearTable ($db, $table)
{
	global $sequences;
	if (!$db->query('TRUNCATE TABLE '. $table))
		return FALSE;
	$seq = db_getSequence($db, $table);
	if ($seq)
	{
		if (!$db->query('ALTER SEQUENCE '. $sequences[$table] .' RESTART WITH 1'))
			return FALSE;
	}
	return TRUE;
}
// Retrieves a database's sequence, if it exists
function db_getSequence ($db, $table)
{
	global $sequences;
	// MySQL does not use sequences
	if ($db->getAttribute(PDO::ATTR_DRIVER_NAME) == 'mysql')
		return NULL;
	if (!isset($sequences[$table]))
		return NULL;
	return $sequences[$table];
}
// Modifies the query to limit rows returned
function db_setLimit ($db, $query, $rows, $offset = 0)
{
	switch ($db->getAttribute(PDO::ATTR_DRIVER_NAME))
	{
	case 'mysql':
		if ($offset)
			$query .= ' LIMIT '. $offset .','. $rows;
		else	$query .= ' LIMIT '. $rows;
		break;
	case 'pgsql':
		$query .= ' LIMIT '. $rows;
		if ($offset)
			$query .= ' OFFSET '. $offset;
		break;
	}
	return $query;
}

// evaluate an SQL query, return first cell of first row
// useful for "SELECT COUNT(*) ..." queries
function sqleval ($db, $query, $parms = NULL)
{
	$q = $db->prepare($query) or warning('SQL prepare failed', 1);
	if ($parms)
		db_bindAllParms($q, $parms);
	$q->execute() or warning('SQL execute failed', 1);
	return $q->fetchColumn();
}

// 1-liner for executing quick SQL queries, intended only for turns.php
function sqlquery ($db, $query, $parms = NULL)
{
	$q = $db->prepare($query) or warning('SQL prepare failed', 1);
	if ($parms)
		db_bindAllParms($q, $parms);
	$q->execute() or warning('SQL execute failed', 1);
	return $q;
}
?>
